@@ -1,6 +1,6 @@
 /* Plzip - Massively parallel implementation of lzip
    Copyright (C) 2009 Laszlo Ersek.
-   Copyright (C) 2009-2021 Antonio Diaz Diaz.
+   Copyright (C) 2009-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 /* This code is based on a patch by Hannes Domani, <ssbssa@yahoo.de> to make
    possible compiling plzip under MS Windows (with MINGW compiler).
 */
-#if defined(__MSVCRT__) && defined(WITH_MINGW)
+#if defined __MSVCRT__ && defined WITH_MINGW
 #include <windows.h>
 #warning "Parallel I/O is not guaranteed to work on Windows."
 
@@ -76,8 +76,8 @@ ssize_t pwrite( int fd, const void *buf, size_t count, uint64_t offset )
 #endif	// __MSVCRT__
 
 
-/* Returns the number of bytes really read.
-   If (returned value < size) and (errno == 0), means EOF was reached.
+/* Return the number of bytes really read.
+   If (value returned < size) and (errno == 0), means EOF was reached.
 */
 int preadblock( const int fd, uint8_t * const buf, const int size,
                 const long long pos )
@@ -96,8 +96,8 @@ int preadblock( const int fd, uint8_t * const buf, const int size,
   }
 
 
-/* Returns the number of bytes really written.
-   If (returned value < size), it is always an error.
+/* Return the number of bytes really written.
+   If (value returned < size), it is always an error.
 */
 int pwriteblock( const int fd, const uint8_t * const buf, const int size,
                  const long long pos )
